package boundedmaxsum;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Map;

import maxsum.discrete.DiscreteInternalFunction;
import maxsum.discrete.DiscreteVariableState;
import maxsum.FixedIterationStoppingCriterion;
import maxsum.Variable;
import maxsum.discrete.DiscreteInternalVariable;
import maxsum.discrete.DiscreteMaxSumController;
import maxsum.discrete.DiscreteMaximiser;
import maxsum.discrete.DiscreteVariable;

import coproblem.COProblem;
import coproblem.MaxSumResult;

public class OptimalUtilityExperiments {
	private static final int REPETITIONS = 25;
	private static final boolean debug = true;

	public static void main(String[] args) {
		Distribution distro = Distribution.GAMMA;
		
		// Perform experiments using random graphs n = number of nodes c =
		// average connections/node
		int col = 3;
		String expType = "adopt";
		//Distribution[] distroList = {Distribution.GAMMA, Distribution.UNIFORM};
		Distribution[] distroList = {Distribution.GAMMA};
		String distroString = ""+distro;
		String filenameOPT = "OPT-"+expType + "-" +distroString + "-" + col + "Col-" + REPETITIONS;
		
		String dataFilePrefix = "/home/sandrof/dataGraphs/AdoptProblems/Problem-GraphColor-";
		if (args.length < 2) {
		for (int j = 0; j < distroList.length; j++) {
			distro = distroList[j];
			distroString = ""+distro;
			for (int c = 2;c<=3;c++){
				filenameOPT = "OPT-"+expType + "-" +distroString + "-" + c + "-" + REPETITIONS;

				BufferedWriter msOut = null;
				try {
					msOut = new BufferedWriter(new FileWriter(filenameOPT
							+ ".dat", true));
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
					//int[] l = {8,10,12,14,16,18,20,25,30,40};
					int[] l = {8,10};
					
					 for (int i = 0; i < l.length; i++) {
						for (int repetitions = 0; repetitions < REPETITIONS; repetitions++) {
								String dataFile = dataFilePrefix + l[i]+"_3_"+c+"_0.4_r"+repetitions; 
								MaxSumResult msres = computeOptimal(dataFile+"-"+distroString+".COP");
								try {
									msOut = new BufferedWriter(new FileWriter(filenameOPT
											+ ".dat", true));
									msOut.write(msres.toCVS()+"\n");
									msOut.close();
								} catch (IOException e) {
									// TODO Auto-generated catch block
									e.printStackTrace();
								}								
						}
					}	 
  		   try {
  			   msOut.close();
  		   } catch (IOException e) {
  			   // TODO Auto-generated catch block
  			   e.printStackTrace();
  		   }
		  }
		}
		} 


	}

	private static MaxSumResult computeOptimal(String fname) {
		COProblem cop = COProblem.load(fname);
		
		DiscreteMaxSumController optimalController = new DiscreteMaxSumController("optimalController"); 		
		
		int nn  = cop.getVars().size();
		int acn = (cop.getFuns().size()/nn);
		
		for (DiscreteInternalFunction f : cop.getFuns()) {
			optimalController.addInternalFunction(f);
		}
		
		for (DiscreteInternalVariable v : cop.getVars()) {
			optimalController.addInternalVariable(v);
		}
		
		
		long start = System.currentTimeMillis();

		
		

		DiscreteMaximiser maximiser = new DiscreteMaximiser(optimalController.getInternalVariables(),
				optimalController.getInternalFunctions());
		Map<DiscreteVariable<?>, DiscreteVariableState> optimalState = maximiser.getOptimalState();

		long end = System.currentTimeMillis();
		
		double u = optimalController.getValue(optimalState); 
		
		
//		optimalController
//		.setStoppingCriterion(new FixedIterationStoppingCriterion(
//				30));
//
//		int i = 0;
//		while (!optimalController.stoppingCriterionIsMet()) {
//			optimalController.calculateNewOutgoingMessages();
//			i++;
//		}		


		long ct = end -start;

		if (debug) {
			System.out.println("running on "+fname);
			System.out.println("utility " + u);
			System.out.println("final state "+optimalState);
		}

		
		MaxSumResult res = new MaxSumResult(nn,acn,u,ct,0);
		
		return res;
	}
}
